"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const inquirer_1 = __importDefault(require("inquirer"));
const command_1 = __importStar(require("@oclif/command"));
const fs_1 = require("fs");
const listr_1 = __importDefault(require("listr"));
const aws_configure_1 = require("../aws-configure");
const trick_registry_1 = require("../tricks/trick-registry");
const shutdown_ec2_instances_trick_1 = require("../tricks/shutdown-ec2-instances.trick");
const stop_fargate_ecs_services_trick_1 = require("../tricks/stop-fargate-ecs-services.trick");
const stop_rds_database_instances_trick_1 = require("../tricks/stop-rds-database-instances.trick");
const decrease_dynamodb_provisioved_rcu_wcu_trick_1 = require("../tricks/decrease-dynamodb-provisioved-rcu-wcu.trick");
const remove_nat_gateways_trick_1 = require("../tricks/remove-nat-gateways.trick");
const snapshot_remove_elasticache_redis_trick_1 = require("../tricks/snapshot-remove-elasticache-redis.trick");
class Conserve extends command_1.default {
    async run() {
        const { flags } = this.parse(Conserve);
        const awsConfig = await aws_configure_1.configureAWS(flags.profile, flags.region);
        await this.validateStateFilePath(flags);
        this.printBanner(awsConfig, flags);
        const tricks = this.getEnabledTricks(flags);
        const taskList = [];
        const stateRoot = {};
        for (const trick of tricks) {
            taskList.push({
                title: `${trick.getDisplayName()}`,
                task: async (ctx, task) => {
                    const subListr = new listr_1.default([], {
                        concurrent: trick.canBeConcurrent(),
                        exitOnError: false,
                        // @ts-ignore
                        collapse: false,
                    });
                    await trick.conserve(subListr, flags['dry-run']).then(result => {
                        stateRoot[trick.getMachineName()] = result;
                        if (!result || (Array.isArray(result) && result.length === 0)) {
                            task.skip('No resources found');
                        }
                    });
                    return subListr;
                },
                // @ts-ignore
                collapse: false,
            });
        }
        await new listr_1.default(taskList, {
            concurrent: true,
            exitOnError: false,
            // @ts-ignore
            collapse: false,
        })
            .run()
            .finally(() => {
            if (!flags['no-state-file']) {
                fs_1.writeFileSync(flags['state-file'], JSON.stringify(stateRoot, null, 2), 'utf-8');
                this.log(`\n  ${chalk_1.default.green('❯')} Wrote state file to ${chalk_1.default.green(flags['state-file'])}`);
            }
        })
            .then(() => {
            if (flags['dry-run']) {
                this.log(`\n${chalk_1.default.yellow(' ↓ Skipped conserve due to dry-run.')}`);
            }
            else {
                this.log(`\n ${chalk_1.default.green('✔')} Successfully conserved.`);
            }
        })
            .catch(error => {
            if (error.errors.length < taskList.length) {
                fs_1.writeFileSync(flags['state-file'], JSON.stringify(stateRoot, null, 2), 'utf-8');
                this.log(`\n${chalk_1.default.yellow('✔')} Partially conserved, with ${chalk_1.default.red(`${error.errors.length} error(s)`)}.`);
            }
            else {
                this.log(`\n${chalk_1.default.red('✖')} All ${chalk_1.default.red(`${taskList.length} tricks failed`)} with errors.`);
            }
        });
    }
    getEnabledTricks(flags) {
        const tricksRegistry = trick_registry_1.TrickRegistry.initialize();
        const tricks = tricksRegistry.all();
        let enabledTricks = flags['no-default-tricks']
            ? []
            : [].concat(...Conserve.tricksEnabledByDefault);
        if (flags['use-trick'] && flags['use-trick'].length > 0) {
            enabledTricks.push(...flags['use-trick']);
        }
        if (flags['ignore-trick'] && flags['ignore-trick'].length > 0) {
            enabledTricks = enabledTricks.filter(disabledTrickName => flags['ignore-trick'].includes(disabledTrickName));
        }
        return enabledTricks.map(trickName => {
            const trick = tricks.find(trick => trick.getMachineName() === trickName);
            if (!trick) {
                this.log(`Could not find a trick named ${chalk_1.default.yellow(trickName)}`);
                this.log(`Run ${chalk_1.default.yellow('aws-cost-saver conserve --help')} to get a list of available tricks.\n`);
                throw new Error('TrickNotFound');
            }
            return trick;
        });
    }
    printBanner(awsConfig, flags) {
        const awsRegion = awsConfig.region || flags.region;
        const awsProfile = awsConfig.credentials.profile || flags.profile;
        this.log(`
AWS Cost Saver
--------------
  Action: ${chalk_1.default.green('conserve')}
  AWS region: ${chalk_1.default.green(awsRegion)}
  AWS profile: ${chalk_1.default.green(awsProfile)}
  State file: ${flags['no-state-file']
            ? chalk_1.default.yellow('none')
            : chalk_1.default.green(flags['state-file'])}
  Dry run: ${flags['dry-run'] ? chalk_1.default.green('yes') : chalk_1.default.yellow('no')}
`);
    }
    async validateStateFilePath(flags) {
        if (!flags['no-state-file'] && fs_1.existsSync(flags['state-file'])) {
            this.log(chalk_1.default.yellow(`\n→ State file already exists: ${chalk_1.default.yellowBright(flags['state-file'])}\n`));
            const answers = await inquirer_1.default.prompt([
                {
                    name: 'stateFileOverwrite',
                    type: 'confirm',
                    default: false,
                    message: `Are you sure you want to ${chalk_1.default.bgRed(chalk_1.default.white('OVERWRITE'))} existing state-file?\n  You will not be able to restore to previously conserved resources!`,
                },
            ]);
            if (!answers.stateFileOverwrite) {
                this.log(chalk_1.default.redBright('\nIgnoring, to avoid overwriting state file!\n'));
                throw new Error('AbortedByUser');
            }
        }
    }
}
exports.default = Conserve;
Conserve.tricksEnabledByDefault = [
    shutdown_ec2_instances_trick_1.ShutdownEC2InstancesTrick.machineName,
    stop_fargate_ecs_services_trick_1.StopFargateEcsServicesTrick.machineName,
    stop_rds_database_instances_trick_1.StopRdsDatabaseInstancesTrick.machineName,
    decrease_dynamodb_provisioved_rcu_wcu_trick_1.DecreaseDynamoDBProvisionedRcuWcuTrick.machineName,
];
Conserve.tricksDisabledByDefault = [
    // Removing NAT gateways can confuse IaC like terraform
    remove_nat_gateways_trick_1.RemoveNatGatewaysTrick.machineName,
    // This is an experimental trick, plus removing and recreating ElastiCache clusters takes a long time
    snapshot_remove_elasticache_redis_trick_1.SnapshotRemoveElasticacheRedisTrick.machineName,
];
Conserve.description = [
    `This command uses various tricks to conserve as much money as possible.`,
    `To restore, this command will create a \`aws-cost-saver.json\` file to be use by "restore".`,
    `\nThese tricks are ${chalk_1.default.bold('enabled')} by default: ${chalk_1.default.green(`\n\t- ${Conserve.tricksEnabledByDefault.join('\n\t- ')}`)}`,
    `\nThese tricks are ${chalk_1.default.bold('disabled')} by default: ${chalk_1.default.redBright(`\n\t- ${Conserve.tricksDisabledByDefault.join('\n\t- ')}`)}`,
].join('\n');
Conserve.examples = [
    `$ aws-cost-saver conserve`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow('--dry-run')}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow('--no-state-file')}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--use-trick ${chalk_1.default.bold(snapshot_remove_elasticache_redis_trick_1.SnapshotRemoveElasticacheRedisTrick.machineName)}`)}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--ignore-trick ${chalk_1.default.bold(stop_rds_database_instances_trick_1.StopRdsDatabaseInstancesTrick.machineName)}`)}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--no-default-tricks --use-trick ${chalk_1.default.bold(stop_fargate_ecs_services_trick_1.StopFargateEcsServicesTrick.machineName)}`)}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--region ${chalk_1.default.bold(`eu-central-1`)} --profile ${chalk_1.default.bold(`my-aws-profile`)}`)}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--state-file ${chalk_1.default.bold(`new-path.json`)}`)}`,
];
Conserve.flags = {
    help: command_1.flags.help({ char: 'h' }),
    region: command_1.flags.string({ char: 'r', default: 'eu-central-1' }),
    profile: command_1.flags.string({ char: 'p', default: 'default' }),
    'dry-run': command_1.flags.boolean({
        char: 'd',
        description: 'Only print actions and write state-file of current resources.',
    }),
    'state-file': command_1.flags.string({
        char: 's',
        default: 'aws-cost-saver.json',
        description: 'Where to keep original state of stopped/decreased resources to restore later.',
    }),
    'no-state-file': command_1.flags.boolean({
        char: 'n',
        default: false,
        description: 'Ignore saving current state, useful when want to only conserve as much money as possible.',
    }),
    'use-trick': command_1.flags.string({
        char: 'u',
        multiple: true,
        description: 'Enables an individual trick. Useful for tricks that are disabled by default.',
    }),
    'ignore-trick': command_1.flags.string({
        char: 'i',
        multiple: true,
        description: 'Disables an individual trick. Useful when you do not like to use a specific trick.',
    }),
    'no-default-tricks': command_1.flags.boolean({
        default: false,
        description: 'Disables all default tricks. Useful alongside --use-trick when you only want a set of specific tricks to execute.',
    }),
};
Conserve.args = [];
//# sourceMappingURL=conserve.js.map