"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs_1 = require("fs");
const trick_registry_1 = require("../tricks/trick-registry");
const aws_configure_1 = require("../aws-configure");
const chalk_1 = __importDefault(require("chalk"));
const listr_1 = __importDefault(require("listr"));
class Restore extends command_1.Command {
    async run() {
        var _a;
        const { flags } = this.parse(Restore);
        const awsConfig = await aws_configure_1.configureAWS(flags.profile, flags.region);
        const awsRegion = awsConfig.region;
        const awsProfile = ((_a = awsConfig.credentials) === null || _a === void 0 ? void 0 : _a.profile) || flags.profile;
        this.log(`
AWS Cost Saver
--------------
  Action: ${chalk_1.default.green('restore')}
  AWS region: ${chalk_1.default.green(awsRegion)}
  AWS profile: ${chalk_1.default.green(awsProfile)}
  State file: ${chalk_1.default.green(flags['state-file'])}
`);
        const tricksRegistry = trick_registry_1.TrickRegistry.initialize();
        const stateContent = fs_1.readFileSync(flags['state-file'], 'utf-8');
        const rootState = JSON.parse(stateContent.toString());
        const taskList = [];
        for (const trick of tricksRegistry.all()) {
            taskList.push({
                title: `${trick.getDisplayName()}`,
                task: async (ctx, task) => {
                    const subListr = new listr_1.default([], {
                        concurrent: false,
                        exitOnError: false,
                        // @ts-ignore
                        collapse: false,
                    });
                    if (rootState[trick.getMachineName()]) {
                        await trick
                            .restore(subListr, flags['dry-run'], rootState[trick.getMachineName()])
                            .catch(task.report);
                    }
                    else {
                        task.skip('Nothing was conserved previously.');
                    }
                    return subListr;
                },
                exitOnError: false,
                // @ts-ignore
                collapse: false,
            });
        }
        await new listr_1.default(taskList, {
            concurrent: true,
            renderer: 'default',
            exitOnError: false,
            // @ts-ignore
            collapse: false,
        })
            .run()
            .then(() => {
            if (flags['dry-run']) {
                this.log(`\n${chalk_1.default.yellow(' ↓ Skipped restore due to dry-run.')}`);
            }
            else {
                this.log(`\n ${chalk_1.default.green('✔')} Successfully restored state from ${chalk_1.default.green(flags['state-file'])}`);
            }
        })
            .catch(error => {
            this.log(`\n${chalk_1.default.yellow('✔')} Partially restored, with ${chalk_1.default.red(`${error.errors.length} errors`)}.`);
        });
    }
}
exports.default = Restore;
Restore.description = 'To restore AWS resources stopped by the conserve command.';
Restore.examples = [
    `$ aws-cost-saver restore`,
    `$ aws-cost-saver restore ${chalk_1.default.yellow('--dry-run')}`,
    `$ aws-cost-saver restore ${chalk_1.default.yellow(`--region ${chalk_1.default.bold(`eu-central-1`)} --profile ${chalk_1.default.bold(`my-aws-profile`)}`)}`,
    `$ aws-cost-saver restore ${chalk_1.default.yellow(`--state-file ${chalk_1.default.bold(`another-path.json`)}`)}`,
];
Restore.flags = {
    help: command_1.flags.help({ char: 'h' }),
    region: command_1.flags.string({ char: 'r', default: 'eu-central-1' }),
    profile: command_1.flags.string({ char: 'p', default: 'default' }),
    'dry-run': command_1.flags.boolean({
        char: 'd',
        description: 'Only print actions but do not do them',
    }),
    'state-file': command_1.flags.string({
        char: 's',
        default: 'aws-cost-saver.json',
        description: 'Path to state-file which contains original state of AWS resources to restore to.',
    }),
};
//# sourceMappingURL=restore.js.map