"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecreaseDynamoDBProvisionedRcuWcuTrick = void 0;
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const chalk_1 = __importDefault(require("chalk"));
class DecreaseDynamoDBProvisionedRcuWcuTrick {
    constructor() {
        this.ddbClient = new aws_sdk_1.default.DynamoDB();
    }
    getMachineName() {
        return DecreaseDynamoDBProvisionedRcuWcuTrick.machineName;
    }
    getDisplayName() {
        return 'Decrease DynamoDB Provisioned RCU and WCU';
    }
    canBeConcurrent() {
        return true;
    }
    async conserve(subListr, dryRun) {
        const tables = await this.listTables();
        const currentState = await this.getCurrentState(tables);
        for (const table of currentState) {
            subListr.add({
                title: chalk_1.default.blueBright(`${table.name}`),
                task: (ctx, task) => this.conserveTable(task, dryRun, table),
            });
        }
        return currentState;
    }
    async restore(subListr, dryRun, originalState) {
        for (const table of originalState) {
            subListr.add({
                title: chalk_1.default.blueBright(table.name),
                task: (ctx, task) => this.restoreTable(task, dryRun, table),
            });
        }
    }
    async conserveTable(task, dryRun, tableState) {
        if (dryRun) {
            task.skip('Skipped due to dry-run');
        }
        else if (tableState.provisionedThroughput) {
            if (tableState.wcu > 1 && tableState.rcu > 1) {
                await this.ddbClient
                    .updateTable({
                    TableName: tableState.name,
                    ProvisionedThroughput: {
                        ReadCapacityUnits: 1,
                        WriteCapacityUnits: 1,
                    },
                })
                    .promise();
                task.output = `Configured RCU = 1 WCU = 1`;
            }
            else {
                task.skip(`Provisioned RCU/WCU is already at minimum of 1`);
            }
        }
        else {
            task.skip(`Provisioned throughput is not configured`);
        }
    }
    async restoreTable(task, dryRun, tableState) {
        if (dryRun) {
            task.skip(`Skipped due to dry-run`);
        }
        else if (tableState.provisionedThroughput) {
            await this.ddbClient
                .updateTable({
                TableName: tableState.name,
                ProvisionedThroughput: {
                    ReadCapacityUnits: tableState.rcu,
                    WriteCapacityUnits: tableState.wcu,
                },
            })
                .promise();
            task.output = `Configured RCU = ${tableState.rcu} WCU = ${tableState.wcu}`;
        }
        else {
            task.skip(`Provisioned throughput was not configured`);
        }
    }
    async getCurrentState(tables) {
        return Promise.all(tables.map(async (table) => {
            var _a;
            const provisionedThroughput = (_a = (await this.ddbClient.describeTable({ TableName: table }).promise()).Table) === null || _a === void 0 ? void 0 : _a.ProvisionedThroughput;
            if (!provisionedThroughput) {
                return {
                    name: table,
                    provisionedThroughput: false,
                    rcu: 0,
                    wcu: 0,
                };
            }
            return {
                name: table,
                provisionedThroughput: true,
                rcu: provisionedThroughput.ReadCapacityUnits || 0,
                wcu: provisionedThroughput.ReadCapacityUnits || 0,
            };
        }));
    }
    async listTables() {
        return (await this.ddbClient.listTables().promise()).TableNames || [];
    }
}
exports.DecreaseDynamoDBProvisionedRcuWcuTrick = DecreaseDynamoDBProvisionedRcuWcuTrick;
DecreaseDynamoDBProvisionedRcuWcuTrick.machineName = 'decrease-dynamodb-provisioned-rcu-wcu';
//# sourceMappingURL=decrease-dynamodb-provisioved-rcu-wcu.trick.js.map