"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShutdownEC2InstancesTrick = void 0;
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const chalk_1 = __importDefault(require("chalk"));
class ShutdownEC2InstancesTrick {
    constructor() {
        this.ec2Client = new aws_sdk_1.default.EC2();
    }
    getMachineName() {
        return ShutdownEC2InstancesTrick.machineName;
    }
    getDisplayName() {
        return 'Shutdown EC2 Instances';
    }
    canBeConcurrent() {
        return false;
    }
    async conserve(subListr, dryRun) {
        const reservations = await this.listReservations();
        const currentState = await this.getCurrentState(reservations);
        for (const instance of currentState) {
            subListr.add({
                title: chalk_1.default.blueBright(`${instance.id} / ${instance.name}`),
                task: (ctx, task) => this.conserveInstance(task, dryRun, instance),
            });
        }
        return currentState;
    }
    async restore(subListr, dryRun, originalState) {
        for (const instance of originalState) {
            subListr.add({
                title: chalk_1.default.blueBright(instance.id),
                task: (ctx, task) => this.restoreInstance(task, dryRun, instance),
            });
        }
    }
    async conserveInstance(task, dryRun, instanceState) {
        if (dryRun) {
            task.skip('Skipped due to dry-run');
        }
        else if (instanceState.state === 'running') {
            await this.ec2Client
                .stopInstances({
                InstanceIds: [instanceState.id],
            })
                .promise();
            task.output = 'Stopped';
        }
        else {
            task.skip(`Not in a "running" state instead in "${instanceState.state}" state`);
        }
    }
    async restoreInstance(task, dryRun, instanceState) {
        if (dryRun) {
            task.skip(`Skipped due to dry-run`);
        }
        else if (instanceState.state === 'running') {
            await this.ec2Client
                .startInstances({
                InstanceIds: [instanceState.id],
            })
                .promise();
            task.output = `Started`;
        }
        else {
            task.skip(`Was not in a "running" state instead in "${instanceState.state}" state`);
        }
    }
    getCurrentState(reservations) {
        return [].concat(...reservations.map(reservation => {
            var _a;
            return (((_a = reservation.Instances) === null || _a === void 0 ? void 0 : _a.map((instance) => {
                var _a;
                if (!instance.InstanceId ||
                    !instance.State ||
                    !instance.State.Name) {
                    throw new Error(`Unexpected EC2 instance: ${JSON.stringify(instance)}`);
                }
                const nameTag = (_a = instance.Tags) === null || _a === void 0 ? void 0 : _a.filter(t => { var _a; return ((_a = t.Key) === null || _a === void 0 ? void 0 : _a.toString().toLowerCase()) === 'name'; }).map(t => t.Value).join(' ');
                return {
                    id: instance.InstanceId,
                    name: nameTag || '<-no name->',
                    state: instance.State.Name,
                };
            })) || []);
        }));
    }
    async listReservations() {
        return ((await this.ec2Client.describeInstances({}).promise()).Reservations || []);
    }
}
exports.ShutdownEC2InstancesTrick = ShutdownEC2InstancesTrick;
ShutdownEC2InstancesTrick.machineName = 'shutdown-ec2-instances';
//# sourceMappingURL=shutdown-ec2-instances.trick.js.map